from cpl_cli.abc.generate_schematic_abc import GenerateSchematicABC


class DBTable(GenerateSchematicABC):
    def __init__(self, *args: str):
        GenerateSchematicABC.__init__(self, *args)
        self._name = self._name.replace("_db_table", "")
        self._class_name = self._class_name.split("Db_table")[0]

    def get_code(self) -> str:
        import textwrap

        code = textwrap.dedent(
            """\
                from datetime import datetime

                from cpl_core.database import TableABC
                
                
                class $ClassName(TableABC):
                    def __init__(
                        self,
                        value: str,
                        created_at: datetime = None,
                        modified_at: datetime = None,
                        id=0,
                    ):
                        self._id = id
                        self._value = value
                
                        TableABC.__init__(self)
                        self._created_at = created_at if created_at is not None else self._created_at
                        self._modified_at = modified_at if modified_at is not None else self._modified_at
                
                    @property
                    def value(self) -> str:
                        return self._value
                
                    @value.setter
                    def value(self, value: str):
                        self._value = value
                
                    @staticmethod
                    def get_select_all_string() -> str:
                        return str(
                            f\"""
                                SELECT * FROM `$TableName`;
                            \"""
                        )
                
                    @staticmethod
                    def get_select_by_id_string(id: int) -> str:
                        return str(
                            f\"""
                                SELECT * FROM `$TableName`
                                WHERE `Id` = {id};
                            \"""
                        )
                
                    @property
                    def insert_string(self) -> str:
                        return str(
                            f\"""
                                INSERT INTO `$TableName` (
                                    `Value`
                                ) VALUES (
                                    {self._value}
                                );
                            \"""
                        )
                
                    @property
                    def udpate_string(self) -> str:
                        return str(
                            f\"""
                                UPDATE `$TableName`
                                SET `Value` = {self._value}
                                WHERE `Id` = {self._id};
                            \"""
                        )
                
                    @property
                    def delete_string(self) -> str:
                        return str(
                            f\"""
                                DELETE FROM `$TableName`
                                WHERE `Id` = {self._id};
                            \"""
                        )
            """
        )
        return self.build_code_str(
            code,
            ClassName=self._class_name,
            TableName=self._class_name,
        )

    @classmethod
    def register(cls):
        GenerateSchematicABC.register(cls, "db-table", [])
