from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.migration_abc import MigrationABC
from bot_data.db_context import DBContext


class SteamSpecialOfferMigration(MigrationABC):
    name = "1.2.0_SteamSpecialOfferMigration"

    def __init__(self, logger: DatabaseLogger, db: DBContext):
        MigrationABC.__init__(self)
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def upgrade(self):
        self._logger.debug(__name__, "Running upgrade")

        self._cursor.execute(
            str(
                f"""
                    CREATE TABLE IF NOT EXISTS `SteamSpecialOffers` (
                        `Id` BIGINT NOT NULL AUTO_INCREMENT,
                        `Game` VARCHAR(255) NOT NULL,
                        `OriginalPrice` FLOAT NOT NULL,
                        `DiscountPrice` FLOAT NOT NULL,
                        `DiscountPct` BIGINT NOT NULL,
                        `CreatedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6),
                        `LastModifiedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6),
                        PRIMARY KEY(`Id`)
                    );
                """
            )
        )

        self._cursor.execute(
            str(
                f"""
                    ALTER TABLE CFG_Server
                    ADD COLUMN IF NOT EXISTS GameOfferNotificationChatId BIGINT NULL AFTER ShortRoleNameSetOnlyHighest;
                """
            )
        )

        self._cursor.execute(
            str(
                f"""
                    ALTER TABLE CFG_ServerHistory
                    ADD COLUMN IF NOT EXISTS GameOfferNotificationChatId BIGINT NULL AFTER ShortRoleNameSetOnlyHighest;
                """
            )
        )

    def downgrade(self):
        self._cursor.execute("DROP TABLE `SteamSpecialOffers`;")
        self._cursor.execute(
            str(
                f"""
                    ALTER TABLE CFG_Server DROP COLUMN ShortRoleNameSetOnlyHighest;
                """
            )
        )
        self._cursor.execute(
            str(
                f"""
                    ALTER TABLE CFG_ServerHistory DROP COLUMN ShortRoleNameSetOnlyHighest;
                """
            )
        )
