from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.api_key_repository_abc import ApiKeyRepositoryABC
from bot_data.abc.game_server_repository_abc import GameServerRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.game_server import GameServer


class GameServerRepositoryService(GameServerRepositoryABC):
    def __init__(
        self,
        logger: DatabaseLogger,
        db_context: DatabaseContextABC,
        servers: ServerRepositoryABC,
        api_keys: ApiKeyRepositoryABC,
    ):
        self._logger = logger

        self._context = db_context
        self._servers = servers
        self._api_keys = api_keys

        GameServerRepositoryABC.__init__(self)

    def _from_result(self, result: tuple):
        return GameServer(
            result[1],
            self._servers.get_server_by_id(result[2]),
            self._api_keys.get_api_key_by_id(result[3]),
            result[4],
            result[5],
            id=result[0],
        )

    def get_game_servers(self) -> List[GameServer]:
        game_servers = List(GameServer)
        self._logger.trace(
            __name__,
            f"Send SQL command: {GameServer.get_select_all_string()}",
        )
        results = self._context.select(GameServer.get_select_all_string())
        for result in results:
            self._logger.trace(__name__, f"Get user-joined-game-server with id {result[0]}")
            game_servers.append(self._from_result(result))

        return game_servers

    def get_game_servers_by_server_id(self, id: int) -> List[GameServer]:
        game_servers = List(GameServer)
        self._logger.trace(
            __name__,
            f"Send SQL command: {GameServer.get_select_by_server_id_string(id)}",
        )
        results = self._context.select(GameServer.get_select_by_server_id_string(id))
        for result in results:
            self._logger.trace(__name__, f"Get user-joined-game-server with id {result[0]}")
            game_servers.append(self._from_result(result))

        return game_servers

    def get_game_server_by_id(self, id: int) -> GameServer:
        self._logger.trace(
            __name__,
            f"Send SQL command: {GameServer.get_select_by_id_string(id)}",
        )
        result = self._context.select(GameServer.get_select_by_id_string(id))[0]
        return self._from_result(result)

    def get_game_servers_by_api_key_id(self, id: int) -> List[GameServer]:
        self._logger.trace(
            __name__,
            f"Send SQL command: {GameServer.get_select_by_api_key_id_string(id)}",
        )
        game_servers = List(GameServer)
        results = self._context.select(GameServer.get_select_by_api_key_id_string(id))
        for result in results:
            game_servers.append(self._from_result(result))
        return game_servers

    def add_game_server(self, game_server: GameServer):
        self._logger.trace(__name__, f"Send SQL command: {game_server.insert_string}")
        self._context.cursor.execute(game_server.insert_string)

    def update_game_server(self, game_server: GameServer):
        self._logger.trace(__name__, f"Send SQL command: {game_server.udpate_string}")
        self._context.cursor.execute(game_server.udpate_string)

    def delete_game_server(self, game_server: GameServer):
        self._logger.trace(__name__, f"Send SQL command: {game_server.delete_string}")
        self._context.cursor.execute(game_server.delete_string)
