from cpl_core.dependency_injection import ServiceProviderABC
from cpl_query.extension import List

from bot_data.model.user_warnings import UserWarnings
from bot_graphql.abc.filter_abc import FilterABC


class UserWarningFilter(FilterABC):
    def __init__(
        self,
        services: ServiceProviderABC,
    ):
        FilterABC.__init__(self)

        self._services = services

        self._id = None
        self._user = None
        self._description = None
        self._author = None

    def from_dict(self, values: dict):
        if "id" in values:
            self._id = int(values["id"])

        if "user" in values:
            from bot_graphql.filter.user_filter import UserFilter

            self._user: UserFilter = self._services.get_service(UserFilter)
            self._user.from_dict(values["user"])

        if "description" in values:
            self._description = values["description"]

        if "author" in values:
            from bot_graphql.filter.user_filter import UserFilter

            self._author: UserFilter = self._services.get_service(UserFilter)
            self._author.from_dict(values["author"])

    def filter(self, query: List[UserWarnings]) -> List[UserWarnings]:
        if self._id is not None:
            query = query.where(lambda x: x.id == self._id)

        if self._user is not None:
            users = self._user.filter(query.select(lambda x: x.user)).select(lambda x: x.id)
            query = query.where(lambda x: x.id in users)

        if self._description is not None:
            query = query.where(lambda x: x.description == self._description or self._description in x.description)

        if self._author is not None:
            users = self._author.filter(query.select(lambda x: x.author)).select(lambda x: x.id)
            query = query.where(lambda x: x.id in users)

        return query
