from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_discord.discord_event_types_enum import DiscordEventTypesEnum
from cpl_discord.service.discord_collection_abc import DiscordCollectionABC

from bot_core.abc.module_abc import ModuleABC
from bot_core.abc.task_abc import TaskABC
from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from modules.base.birthday_watcher import BirthdayWatcher
from modules.base.command.afk_command import AFKCommand
from modules.base.command.game_server_group import GameServerGroup
from modules.base.command.help_command import HelpCommand
from modules.base.command.info_command import InfoCommand
from modules.base.command.make_coffee_command import MakeCoffeeCommand
from modules.base.command.mass_move_command import MassMoveCommand
from modules.base.command.ping_command import PingCommand
from modules.base.command.presence_command import PresenceCommand
from modules.base.command.purge_command import PurgeCommand
from modules.base.command.register_group import RegisterGroup
from modules.base.command.submit_group import SubmitGroup
from modules.base.command.unregister_group import UnregisterGroup
from modules.base.command.user_group import UserGroup
from modules.base.events.base_on_command_error_event import BaseOnCommandErrorEvent
from modules.base.events.base_on_command_event import BaseOnCommandEvent
from modules.base.events.base_on_guild_join_event import BaseOnGuildJoinEvent
from modules.base.events.base_on_member_join_event import BaseOnMemberJoinEvent
from modules.base.events.base_on_member_remove_event import BaseOnMemberRemoveEvent
from modules.base.events.base_on_message_delete_event import BaseOnMessageDeleteEvent
from modules.base.events.base_on_message_event import BaseOnMessageEvent
from modules.base.events.base_on_raw_reaction_add import BaseOnRawReactionAddEvent
from modules.base.events.base_on_raw_reaction_remove import BaseOnRawReactionRemoveEvent
from modules.base.events.base_on_scheduled_event_update_event import (
    BaseOnScheduledEventUpdateEvent,
)
from modules.base.events.base_on_voice_state_update_event import (
    BaseOnVoiceStateUpdateEvent,
)
from modules.base.events.base_on_voice_state_update_event_help_channel import (
    BaseOnVoiceStateUpdateEventHelpChannel,
)
from modules.base.events.base_on_voice_state_update_event_scheduled_event_bonus import (
    BaseOnVoiceStateUpdateEventScheduledEventBonus,
)
from modules.base.forms.bug_report_form import BugReportForm
from modules.base.forms.complaint_form import ComplaintForm
from modules.base.helper.base_reaction_handler import BaseReactionHandler
from modules.base.service.event_service import EventService
from modules.base.service.user_warnings_service import UserWarningsService


class BaseModule(ModuleABC):
    def __init__(self, dc: DiscordCollectionABC):
        ModuleABC.__init__(self, dc, FeatureFlagsEnum.base_module)

    def configure_configuration(self, config: ConfigurationABC, env: ApplicationEnvironmentABC):
        pass

    def configure_services(self, services: ServiceCollectionABC, env: ApplicationEnvironmentABC):
        services.add_transient(BaseReactionHandler)
        services.add_singleton(EventService)
        services.add_transient(UserWarningsService)
        services.add_singleton(TaskABC, BirthdayWatcher)

        # forms
        services.add_transient(BugReportForm)
        services.add_transient(ComplaintForm)

        # commands
        services.add_transient(AFKCommand)
        services.add_transient(SubmitGroup)
        services.add_transient(HelpCommand)
        services.add_transient(InfoCommand)
        services.add_transient(MassMoveCommand)
        services.add_transient(PingCommand)
        services.add_transient(PresenceCommand)
        services.add_transient(MakeCoffeeCommand)

        services.add_transient(PurgeCommand)
        services.add_transient(UserGroup)
        services.add_transient(RegisterGroup)
        services.add_transient(UnregisterGroup)
        services.add_transient(GameServerGroup)
        # events
        services.add_transient(DiscordEventTypesEnum.on_command.value, BaseOnCommandEvent)
        services.add_transient(DiscordEventTypesEnum.on_command_error.value, BaseOnCommandErrorEvent)
        services.add_transient(DiscordEventTypesEnum.on_member_join.value, BaseOnMemberJoinEvent)
        services.add_transient(DiscordEventTypesEnum.on_member_remove.value, BaseOnMemberRemoveEvent)
        services.add_transient(DiscordEventTypesEnum.on_message.value, BaseOnMessageEvent)
        services.add_transient(DiscordEventTypesEnum.on_message_delete.value, BaseOnMessageDeleteEvent)
        services.add_transient(DiscordEventTypesEnum.on_raw_reaction_add.value, BaseOnRawReactionAddEvent)
        services.add_transient(
            DiscordEventTypesEnum.on_raw_reaction_remove.value,
            BaseOnRawReactionRemoveEvent,
        )
        services.add_transient(
            DiscordEventTypesEnum.on_voice_state_update.value,
            BaseOnVoiceStateUpdateEvent,
        )
        services.add_transient(
            DiscordEventTypesEnum.on_voice_state_update.value,
            BaseOnVoiceStateUpdateEventHelpChannel,
        )
        services.add_transient(
            DiscordEventTypesEnum.on_voice_state_update.value,
            BaseOnVoiceStateUpdateEventScheduledEventBonus,
        )
        services.add_transient(
            DiscordEventTypesEnum.on_scheduled_event_update.value,
            BaseOnScheduledEventUpdateEvent,
        )
        services.add_transient(
            DiscordEventTypesEnum.on_guild_join.value,
            BaseOnGuildJoinEvent,
        )
