import { Component, OnInit } from "@angular/core";
import { Router } from "@angular/router";
import { TranslateService } from "@ngx-translate/core";
import { MenuItem, PrimeNGConfig } from "primeng/api";
import { SettingsService } from "src/app/services/settings/settings.service";
import { SpinnerService } from "src/app/services/spinner/spinner.service";
import { ThemeService } from "src/app/services/theme/theme.service";

@Component({
  selector: 'app-auth-header',
  templateUrl: './auth-header.component.html',
  styleUrls: ['./auth-header.component.scss']
})
export class AuthHeaderComponent implements OnInit {
  public langList: MenuItem[] = [];
  public themeList: MenuItem[] = [];

  constructor(
    private router: Router,
    private themeService: ThemeService,
    private spinnerService: SpinnerService,
    private settings: SettingsService,
    private translateService: TranslateService,
    private config: PrimeNGConfig
  ) { }

  public ngOnInit(): void {
    this.initMenuLists();
    this.loadLang();
  }

  private initMenuLists(): void {
    this.langList = [
      {
        label: 'English', command: () => {
          this.translate('en');
          this.setLang('en');
        },
      },
      {
        label: 'Deutsch', command: () => {
          this.translate('de');
          this.setLang('de');
        },
      },
    ];

    this.settings.getThemes()?.forEach(theme => {
      this.themeList.push({
        label: theme.Label,
        command: () => {
          this.changeTheme(theme.Name);
        }
      });
    });
  }

  private changeTheme(name: string): void {
    this.themeService.setTheme(name, true);
  }

  private translate(lang: string) {
    this.translateService.use(lang);
    this.translateService.get('primeng').subscribe(res => this.config.setTranslation(res));
  }

  private loadLang(): void {
    let lang = localStorage.getItem(`default_lang`);
    if (!lang) {
      lang = 'en';
      this.setLang(lang);
    }
    this.translate(lang);
  }

  private setLang(lang: string): void {
    localStorage.setItem(`default_lang`, lang);
  }

}
